#define _CRT_SECURE_NO_WARNINGS
#include <cassert>
#include <cstdio>
#include <cstdint>
#include <ctime>
#include <sys/types.h>
#include <sys/timeb.h>
#include <Windows.h>

#include "bb_api.h"
#pragma comment(lib,"bb_api.lib")

int main(int argc, char **argv)
{
    // Open the device as usual 
    int handle;
    bbOpenDevice(&handle);
    // Configure an IQ data stream as usual 
    bbConfigureCenterSpan(handle, 2400.0e6, 20.0e6); 
    bbConfigureLevel(handle, -20.0, BB_AUTO_ATTEN); 
    bbConfigureGain(handle, BB_AUTO_GAIN);
    bbConfigureIQ(handle, 4, 6.0e6);
    // Configure the device to accept input triggers on port 2 
    // The 1 PPS trigger will be connected to port 2 
    bbConfigureIO(handle, 0, BB_PORT2_IN_TRIGGER_RISING_EDGE);
    // At this point the GPS receiver must be operational 
    // The RS232 connection cannot be open, and the COM port 
    // and the baud rate must be known 
    // Ensure the receiver is locked 
    if(bbSyncCPUtoGPS(3, 38400) != bbNoError) {
        printf("Error configuring COM port");
        assert(false);
        return -1;
    }
    // If syncCPUtoGPS returned successfully the device can now be initialized 
    // and the RS232 connection should now be closed. 
    // Note: BB_TIME_STAMP is required so the device treats input triggers as the 
    // GPS 1 PPS 
    bbInitiate(handle, BB_STREAMING, BB_STREAM_IQ | BB_TIME_STAMP);

    // Verify bandwidth and sample rate
    double bandwidth;
    int sampleRate;
    bbQueryStreamInfo(handle, 0, &bandwidth, &sampleRate);

    // Allocate memory for IQ values
    const int BLOCK_SIZE = 32768;
    float *buffer = new float[BLOCK_SIZE * 2];

    // Set up block capture
    bbIQPacket pkt;
    pkt.iqData = buffer;
    pkt.iqCount = BLOCK_SIZE;
    pkt.triggers = 0;
    pkt.triggerCount = 0;
    // Setting purge to true tells the API to return IQ data that has been
    // acquired only after calling the bbGetIQ() function. Setting purge to false
    // will return data contiguous to the data returned from the last call to
    // bbGetIQ().
    pkt.purge = false;

    // Wait for first PPS trigger to occur in the API
    // Until the first PPS trigger arrives at the device, it is not possible for the IQ data time
    //  stamps to be accurate. 
    Sleep(1500);

    int numSecsPassed = 0;
    int lastSec = 0;
    while(numSecsPassed < 20) {
        bbGetIQ(handle, &pkt);

        if(pkt.sec != lastSec) {
            lastSec = pkt.sec;
            numSecsPassed++;
            printf("Seconds since epoch: %d.%d\n", pkt.sec, pkt.nano);

            time_t t = pkt.sec;
            tm tm;
            localtime_s(&tm, &t);
            printf("Date/Time: %s\n", asctime(&tm));
        }
    }

    // Done 
    bbCloseDevice(handle);
    return 0;
}